package ASCII::Image;

=pod

=head1 NAME

ASCII::Image - image utility routines 

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw(
make_color
);

use Carp qw( carp confess croak );
use FindBin;
use Math::VecStat qw(min max);
use Memoize;

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;
use ASCII::Timer;
use ASCII::Util;
use Imager;

################################################################
#
# Assign values to each pixel of the image. The image is
# either the input image or a character glyph.
#
# Only greyscale images are supported - red channel is used as signal value, which
# is then mapped onto [0,1] where 0 = background, 1 = foreground

sub get_pixels { 
    start_timer("get_pixels");
    my $image      = shift;
    my $tone_curve = shift || 1;
    my $cutoff     = shift || 0;
    my $data;
    my ($sum,$n) = (0,0);
    printdebug(2,"fetching pixels values from",$image->{tag});
    for my $y (0..$image->{h}-1) {
	my @colors = map { [$_->rgba()] } $image->{img}->getscanline("y"=>$y);
	for my $x (0..$image->{w}-1) {
	    my ($r,$g,$b,$alpha) = @{$colors[$x]};
	    my $value            = min($r,$g,$b);
	    my $value_remap      = (255-$value)/255;
	    $value_remap         = 0 if $value_remap < $cutoff;
	    $image->{data}->[$x][$y] = $value_remap;
	    $sum += $value_remap ** $tone_curve;
	    $n++;
	}
    }
    $image->{avg} = $n ? $sum/$n : 0;
    stop_timer("get_pixels");
}

sub make_color {
    my ($str,$alpha) = @_;
    $alpha = 255 if ! defined $alpha;
    printdebug(2,"created color",$str);
    return Imager::Color->new(web=>"#$str",alpha=>$alpha);
}

1;
