package ASCII::SVG;

=pod

=head1 NAME

ASCII::Font - utility routines for font handling 

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw();

use Carp qw( carp confess croak );
use FindBin;
use XML::Code;

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;
use ASCII::Timer;
use ASCII::Util;

our $fh;
our $make_svg;

sub init_svg {
    $make_svg = not_defined_or_true($main::CONF{output}{svg});
    return if ! $make_svg;
    my $file = $main::CONF{output}{image};
    $fh = IO::File->new(">$file.svg");
}

sub generate_svg {
    return if ! $make_svg;
    start_timer("svg");
    my ($layer,$fg,@grow) = @_;
    my @grow_svg_groups = ASCII::SVG::make_svg_groups(@grow);
    for my $group (@grow_svg_groups) {
	my $xml = XML::Code->new("=");
	$xml->set_text($group->{str});
	my $str = $xml->code();
	$str = $group->{str} if $str eq "";
	next if $str eq " ";
	svg_start_group("font-size"=>$layer->{size},
			"transform"=>sprintf("translate(%s,%s)",
					     $group->{x},
					     $group->{y}));
	svg_text($str,
		 $group->{x},
		 $group->{y},
		 $group->{font_name},
		 $layer->{size},
		 $fg->rgba());
	svg_end_group();
    }
    stop_timer("svg");
}

sub make_svg_groups {
    return if ! $make_svg;
    my @grow = @_;
    my @groups;
    my $curr_font;
    for my $g (@grow) {
	if(1 || ! $curr_font || $g->{font_file} ne $curr_font) {
	    push @groups, {font_file => $g->{font_file},
			   font_name => $g->{font_name},
			   "x" => $g->{x},
			   "y" => $g->{y} + $g->{ascent},
			   str => $g->{c}};
	} else {
	    $groups[-1]{str} .= $g->{c};
	}
	$curr_font = $g->{font_file};
    }
    return @groups;
}

sub svg_text {
    return if ! $make_svg;
    my ($text,$x,$y,$font,$size,$r,$g,$b) = @_;
    printf $fh (qq{<text font-family="%s" fill="rgb(%d,%d,%d)">%s</text>\n},
		$font,
		$r,$g,$b,
	      $text);
}

sub svg_start_group {
    return if ! $make_svg;
    my %args = @_;
    my @args;
    for my $key (keys %args) {
	push @args, sprintf(qq{%s="%s"},$key,$args{$key});
    }
    print $fh sprintf(qq{<g %s>},join(" ",@args));
}

sub svg_end_group {
    return if ! $make_svg;
    print $fh "</g>\n";
}

sub svg_preamble {
    return if ! $make_svg;
    my $text=<<END;
<?xml version="1.0" encoding="utf-8"?>
<!-- Generator: asciifyimage (c) Martin Krzywinski mkweb.bcgsc.ca/asciiart -->
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
END
print $fh $text;
}

sub svg_postamble {
    return if ! $make_svg;
  my $text=<<END;
</svg>
END
print $fh "$text\n";
}

sub svg_canvas {
    return if ! $make_svg;
    my ($w,$h) = @_;
    my $text=<<END;
    <svg version="1.1" id="Layer1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="${w}px" height="${h}px" xml:space="preserve">
END
print $fh "$text\n";
}

1;
