package ASCII::Text;

=pod

=head1 NAME

ASCII::Text - text handling routines 

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw(

);

use Carp qw( carp confess croak );
use Data::Dumper;
use List::MoreUtils qw(uniq);
use FindBin;

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;
use ASCII::Util;

sub get_characters {
    my $layer = shift;
    my @chars;
    my $set_name = $layer->{characterset_name} || $main::CONF{layers}{characterset_name} || $main::CONF{charactersets}{characterset_name};
    my $set      = $main::CONF{charactersets}{characterset}{$set_name} || confess "character set [$set_name] not defined";
    printdebug(1,"using character set",$set_name);
    if($set->{file}) {
	printdebug(1,"characters from file",$set->{file});
	@chars = get_characters_from_file($set->{file});
    } elsif ($set->{str}) {
	printdebug(1,"characters from string",$set->{str});
	@chars = split("",$set->{str});
    } elsif ($set->{eval}) {
	printdebug(1,"characters from eval",$set->{eval});
	@chars = eval $set->{eval};
    }
    my $transform_name = first_defined($layer->{transform_name},
				       $set->{transform_name},
				       $main::CONF{layers}{transform_name},
				       $main::CONF{transforms}{transform_name});

    my $transform = $main::CONF{transforms}{transform}{$transform_name} || confess "text transform [$transform_name] not defined";
    printdebug(1,"using transform",$transform_name);
    if($transform->{case}) {
	if($transform->{case} eq "lower") {
	    printdebug(1,"transform: lower case");
	    @chars = map { lc } @chars;
	}
	if($transform->{case} eq "upper") {
	    printdebug(1,"transform: upper case");
	    @chars = map { uc } @chars;
	}
    }
    if($transform->{add_space}) {
	printdebug(1,"transform: add space");
	push @chars, " ";
    }
    if($transform->{reject_space}) {
	printdebug(1,"transform: reject space");
	@chars = grep($_ ne " ", @chars);
    }
    if($transform->{unique} || $transform->{uniq}) {
	printdebug(1,"transform: unique only");
	@chars = uniq @chars;
    }
    if($transform->{sort}) {
	if($transform->{sort} =~ /asc/) {
	    printdebug(1,"transform: asc sort");
	    @chars = sort {$a cmp $b} @chars;
	} elsif($transform->{sort} =~ /desc/) {
	    printdebug(1,"transform: desc sort");
	    @chars = sort {$b cmp $a} @chars;
	} else {
	    die "do not understand sort [$transform->{sort}]";
	}
    }
    if($transform->{rx}) {
	if($transform->{rx}{reject}) {
	    printdebug(1,"transform: rx filter reject",$transform->{rx}{reject});
	    @chars = grep($_ !~ /$transform->{rx}{reject}/, @chars) if $transform->{rx}{reject};
	}
	if($transform->{rx}{accept}) {
	    printdebug(1,"transform: rx filter accept",$transform->{rx}{accept});
	    @chars = grep($_ =~ /$transform->{rx}{accept}/, @chars) if $transform->{rx}{accept};
	}
    }
    return ($set,$transform,@chars);
}

sub get_characters_from_file {
    my $file = shift;
    open(F,$file) || confess "cannot open character file [$file]";
    my @chars;
    while(<F>) {
	chomp;
	# collapse multiple space characters to a single space
	s/\s+/ /g;
	push @chars, split("",$_);
    }
    return @chars;
}

1;
