package ASCII::Timer;

=pod

=head1 NAME

ASCII::Timer - utility routines for timing parts of the code

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw(
start_timer
stop_timer
report_timers
get_timer
);

use Carp qw( carp confess croak );
use Data::Dumper;
use FindBin;
use Time::HiRes qw(gettimeofday tv_interval);

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;
use ASCII::Util;

our $timers;

sub start_timer {
    my $name = shift;
    $timers->{$name}{t} = [gettimeofday];
    $timers->{$name}{running} = 1;
} 
sub stop_timer {
    my $name = shift;
    $timers->{$name}{elapsed} += tv_interval($timers->{$name}{t});
    $timers->{$name}{running} = 0;
}
sub get_timer {
    my $name = shift;
    return $timers->{$name}{elapsed};
}
sub report_timers {
    for my $timer (keys %$timers) {
	if($timers->{$timer}{running}) {
	    stop_timer($timer);
	}
    }
    for my $timer (sort {$timers->{$b}{elapsed} <=> $timers->{$a}{elapsed}} keys %$timers) {
	printinfo(sprintf("timer %-12s %7.3f %4.2f",
			  $timer,
			  $timers->{$timer}{elapsed},
			  100*$timers->{$timer}{elapsed} / $timers->{all}{elapsed}));
    }
}

1;
