package ASCII::Util;

=pod

=head1 NAME

ASCII::Util - utility routines 

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw(
fetch_config
to_list
dim
defined_and_false
not_defined_or_true
first_defined
);

use Carp qw( carp confess croak );
use FindBin;
use Memoize;

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;

#map { memoize $_ } qw(fetch_config);

sub first_defined {
    my ($x) = grep(defined $_, @_);
    return $x;
}
sub fetch_config {
    my ($param,@paths) = @_;
    for my $path (@paths) {
	return $path->{$param} if exists $path->{$param};
    }
    confess "Cannot find parameter value [$param]";
    return;
}

sub to_list {
    my $x = shift;
    return ref($x) eq "ARRAY" ? @$x : ($x);
}

sub dim {
    return sprintf("%d x %d",@_);
}

sub defined_and_false {
    my $x = shift;
    return 1 if defined $x && ! $x;
}

sub not_defined_or_true {
    my $x = shift;
    return 1 if ! defined $x || $x;
}

1;
